# pylint: disable=line-too-long
"""The save commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SAVe:EVENTtable:BUS<x> <string>
    - SAVe:MARKS <string>
    - SAVe:MASK <QString>
    - SAVe:SETUp {<file path>|<NR1>}
    - SAVe:WAVEform [<wfm>,REF<x>]| [<wfm>,<QString>] | [ALL,[QString]]| [DIGITALALL,<QString>]
    - SAVe:WAVEform:DATa:STARt {<NR1>}
    - SAVe:WAVEform:DATa:STOP {<NR1>}
    - SAVe:WAVEform:FILEFormat {INTERNal|MATHCad|MATLab|SPREADSHEETCsv| SPREADSHEETTxt|TIMEStamp|AUTO}
    - SAVe:WAVEform:FILEFormat?
    - SAVe:WAVEform:FORCESAMEFilesize {<NR1>|OFF|ON}
    - SAVe:WAVEform:FORCESAMEFilesize?
"""  # noqa: E501
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SaveWaveformForcesamefilesize(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform:FORCESAMEFilesize`` command.

    **Description:**
        - This command sets or queries the save waveform force same file size.

    **Usage:**
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:FORCESAMEFilesize?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FORCESAMEFilesize?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:FORCESAMEFilesize value``
          command.

    **SCPI Syntax:**

    ::

        - SAVe:WAVEform:FORCESAMEFilesize {<NR1>|OFF|ON}
        - SAVe:WAVEform:FORCESAMEFilesize?

    **Info:**
        - ``<NR1>`` = 0 disables the function; any other value enables it.
        - ``OFF`` disables the function.
        - ``ON`` enables the function.
    """


class SaveWaveformFileformat(SCPICmdWrite, SCPICmdRead):
    r"""The ``SAVe:WAVEform:FILEFormat`` command.

    **Description:**
        - This command specifies or returns the file format for saved waveforms. Waveform header and
          timing information is included in the resulting file of non-internal formats. The
          instrument saves DPO waveforms as a 1000 x 502 matrix, with the first row corresponding to
          the most recently acquired data. The values specified by ``SAVE:WAVEFORM:DATA:START`` and
          ``SAVE:WAVEFORM:DATA:STOP`` determine the range of waveform data to output. In the event
          that ``SAVe:WAVEform:DATa:STOP`` value is greater than the current record length, the
          current record length determines the last output value. This command is equivalent to
          selecting Save As from the File menu, clicking the Waveform button, and selecting the
          desired waveform file format.

    **Usage:**
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FILEFormat?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:FILEFormat value``
          command.

    **SCPI Syntax:**

    ::

        - SAVe:WAVEform:FILEFormat {INTERNal|MATHCad|MATLab|SPREADSHEETCsv| SPREADSHEETTxt|TIMEStamp|AUTO}
        - SAVe:WAVEform:FILEFormat?

    **Info:**
        - ``INTERNal`` specifies that waveforms are saved in an internal format, using a .wfm file
          name extension. These files can be recalled as reference waveforms. When this argument is
          specified, the settings specified via the ``SAVe:WAVEform:DATa:STARt`` and
          ``SAVe:WAVEform:DATa:STOP`` commands have no meaning as the entire waveform is saved.
        - ``MATHCad`` specifies that waveforms are saved in MathCad format, using a .dat file name
          extension. When saving in this format, waveform values are delimited with new lines.
          MathCad format enables easy import of waveform data into MathCad or MATLAB. For FastAcq
          waveforms, data is imported as a matrix. For these formats, waveform header and timing
          information is saved in a separate header file. MathCad format header files use a _hdr.dat
          extension.
        - ``MATLab`` specifies that waveforms are saved in Matlab format, using a .dat file name
          extension. When saving in this format, waveform values are delimited with new lines.
          MATLAB format enables easy import of waveform data into MathCad or MATLAB. For FastAcq
          waveforms, data is imported as a matrix. For these formats, waveform header and timing
          information is saved in a separate header file. MATLAB format header files use a _hdr.dat
          extension.
        - ``SPREADSHEETCsv`` specifies that waveform data is saved in a format that contains comma
          delimited values. These waveform data files are named using the .csv file name extension.
          Saving waveforms in CSV format enables spreadsheet programs to import the data.
        - ``SPREADSHEETTxt`` specifies that waveform data is saved in a format that contains tab
          delimited values. These waveform data files are named using the .txt file name extension.
          Saving waveforms in this format enables spreadsheet programs to import the data.
        - ``TIMEStamp`` specifies that timestamp data is saved in a format that contains comma
          delimited values. These data files are named using the .txt file name extension. Saving
          waveforms in this format enables spreadsheet programs to import the data.
        - ``AUTO`` specifies that the file format should be taken from the file name extension.
          Supported extensions include \\*.wfm, \\*.csv, \\*.txt, \\*.h5). If an extension is read
          that is not supported, no file will be written out. If no extension is given in the
          filename, \\*.csv will be written out. Auto format always uses the entire waveform record
          and does not utilize ``SAVe:WAVEform:DATa:STARt`` or STOP or any other limitations on
          output data.
    """  # noqa: E501


class SaveWaveformDataStop(SCPICmdWrite):
    """The ``SAVe:WAVEform:DATa:STOP`` command.

    **Description:**
        - Sets or queries the save waveform ending waveform data sample.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:DATa:STOP value``
          command.

    **SCPI Syntax:**

    ::

        - SAVe:WAVEform:DATa:STOP {<NR1>}

    **Info:**
        - ``<NR1>`` specifies the ending waveform sample used by the ``SAVe:WAVEform`` command. This
          works for Spreadsheet CSV, Spreadsheet TXT, Mathcad, and Matlab file formats.
    """


class SaveWaveformDataStart(SCPICmdWrite):
    """The ``SAVe:WAVEform:DATa:STARt`` command.

    **Description:**
        - Sets or queries the save waveform starting waveform data sample.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:DATa:STARt value``
          command.

    **SCPI Syntax:**

    ::

        - SAVe:WAVEform:DATa:STARt {<NR1>}

    **Info:**
        - ``<NR1>`` specifies the starting waveform sample used by the ``SAVe:WAVEform`` command.
          This works for Spreadsheet CSV, Spreadsheet TXT, Mathcad, and Matlab file formats.
    """


class SaveWaveformData(SCPICmdRead):
    """The ``SAVe:WAVEform:DATa`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:DATa?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:DATa?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.start``: The ``SAVe:WAVEform:DATa:STARt`` command.
        - ``.stop``: The ``SAVe:WAVEform:DATa:STOP`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._start = SaveWaveformDataStart(device, f"{self._cmd_syntax}:STARt")
        self._stop = SaveWaveformDataStop(device, f"{self._cmd_syntax}:STOP")

    @property
    def start(self) -> SaveWaveformDataStart:
        """Return the ``SAVe:WAVEform:DATa:STARt`` command.

        **Description:**
            - Sets or queries the save waveform starting waveform data sample.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:DATa:STARt value``
              command.

        **SCPI Syntax:**

        ::

            - SAVe:WAVEform:DATa:STARt {<NR1>}

        **Info:**
            - ``<NR1>`` specifies the starting waveform sample used by the ``SAVe:WAVEform``
              command. This works for Spreadsheet CSV, Spreadsheet TXT, Mathcad, and Matlab file
              formats.
        """
        return self._start

    @property
    def stop(self) -> SaveWaveformDataStop:
        """Return the ``SAVe:WAVEform:DATa:STOP`` command.

        **Description:**
            - Sets or queries the save waveform ending waveform data sample.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:DATa:STOP value``
              command.

        **SCPI Syntax:**

        ::

            - SAVe:WAVEform:DATa:STOP {<NR1>}

        **Info:**
            - ``<NR1>`` specifies the ending waveform sample used by the ``SAVe:WAVEform`` command.
              This works for Spreadsheet CSV, Spreadsheet TXT, Mathcad, and Matlab file formats.
        """
        return self._stop


class SaveWaveform(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform`` command.

    **Description:**
        - This command (no query form) saves a waveform to one of four reference memory locations or
          a file.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform value`` command.

    **SCPI Syntax:**

    ::

        - SAVe:WAVEform [<wfm>,REF<x>]| [<wfm>,<QString>] | [ALL,[QString]]| [DIGITALALL,<QString>]

    **Info:**
        - ``<wfm>,REF<x>`` saves the specified waveform to the specified reference memory location.
        - ``<wfm>,<QString>`` saves the specified waveform to the file specified.
        - ``ALL,[QString>]`` saves all displayed waveforms, excluding digital waveforms, to
          individual files based on the ``SAVE:WAVEFORM:FILEFORMAT`` setting.
        - ``DIGITALALL,<QString>`` saves all digital waveforms to a single file specified by the
          quoted string argument and based on the ``SAVE:WAVEFORM:FILEFORMAT`` setting.

    Properties:
        - ``.data``: The ``SAVe:WAVEform:DATa`` command tree.
        - ``.fileformat``: The ``SAVe:WAVEform:FILEFormat`` command.
        - ``.forcesamefilesize``: The ``SAVe:WAVEform:FORCESAMEFilesize`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._data = SaveWaveformData(device, f"{self._cmd_syntax}:DATa")
        self._fileformat = SaveWaveformFileformat(device, f"{self._cmd_syntax}:FILEFormat")
        self._forcesamefilesize = SaveWaveformForcesamefilesize(
            device, f"{self._cmd_syntax}:FORCESAMEFilesize"
        )

    @property
    def data(self) -> SaveWaveformData:
        """Return the ``SAVe:WAVEform:DATa`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:DATa?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:DATa?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.start``: The ``SAVe:WAVEform:DATa:STARt`` command.
            - ``.stop``: The ``SAVe:WAVEform:DATa:STOP`` command.
        """
        return self._data

    @property
    def fileformat(self) -> SaveWaveformFileformat:
        r"""Return the ``SAVe:WAVEform:FILEFormat`` command.

        **Description:**
            - This command specifies or returns the file format for saved waveforms. Waveform header
              and timing information is included in the resulting file of non-internal formats. The
              instrument saves DPO waveforms as a 1000 x 502 matrix, with the first row
              corresponding to the most recently acquired data. The values specified by
              ``SAVE:WAVEFORM:DATA:START`` and ``SAVE:WAVEFORM:DATA:STOP`` determine the range of
              waveform data to output. In the event that ``SAVe:WAVEform:DATa:STOP`` value is
              greater than the current record length, the current record length determines the last
              output value. This command is equivalent to selecting Save As from the File menu,
              clicking the Waveform button, and selecting the desired waveform file format.

        **Usage:**
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FILEFormat?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:FILEFormat value``
              command.

        **SCPI Syntax:**

        ::

            - SAVe:WAVEform:FILEFormat {INTERNal|MATHCad|MATLab|SPREADSHEETCsv| SPREADSHEETTxt|TIMEStamp|AUTO}
            - SAVe:WAVEform:FILEFormat?

        **Info:**
            - ``INTERNal`` specifies that waveforms are saved in an internal format, using a .wfm
              file name extension. These files can be recalled as reference waveforms. When this
              argument is specified, the settings specified via the ``SAVe:WAVEform:DATa:STARt`` and
              ``SAVe:WAVEform:DATa:STOP`` commands have no meaning as the entire waveform is saved.
            - ``MATHCad`` specifies that waveforms are saved in MathCad format, using a .dat file
              name extension. When saving in this format, waveform values are delimited with new
              lines. MathCad format enables easy import of waveform data into MathCad or MATLAB. For
              FastAcq waveforms, data is imported as a matrix. For these formats, waveform header
              and timing information is saved in a separate header file. MathCad format header files
              use a _hdr.dat extension.
            - ``MATLab`` specifies that waveforms are saved in Matlab format, using a .dat file name
              extension. When saving in this format, waveform values are delimited with new lines.
              MATLAB format enables easy import of waveform data into MathCad or MATLAB. For FastAcq
              waveforms, data is imported as a matrix. For these formats, waveform header and timing
              information is saved in a separate header file. MATLAB format header files use a
              _hdr.dat extension.
            - ``SPREADSHEETCsv`` specifies that waveform data is saved in a format that contains
              comma delimited values. These waveform data files are named using the .csv file name
              extension. Saving waveforms in CSV format enables spreadsheet programs to import the
              data.
            - ``SPREADSHEETTxt`` specifies that waveform data is saved in a format that contains tab
              delimited values. These waveform data files are named using the .txt file name
              extension. Saving waveforms in this format enables spreadsheet programs to import the
              data.
            - ``TIMEStamp`` specifies that timestamp data is saved in a format that contains comma
              delimited values. These data files are named using the .txt file name extension.
              Saving waveforms in this format enables spreadsheet programs to import the data.
            - ``AUTO`` specifies that the file format should be taken from the file name extension.
              Supported extensions include \\*.wfm, \\*.csv, \\*.txt, \\*.h5). If an extension is
              read that is not supported, no file will be written out. If no extension is given in
              the filename, \\*.csv will be written out. Auto format always uses the entire waveform
              record and does not utilize ``SAVe:WAVEform:DATa:STARt`` or STOP or any other
              limitations on output data.
        """  # noqa: E501
        return self._fileformat

    @property
    def forcesamefilesize(self) -> SaveWaveformForcesamefilesize:
        """Return the ``SAVe:WAVEform:FORCESAMEFilesize`` command.

        **Description:**
            - This command sets or queries the save waveform force same file size.

        **Usage:**
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:FORCESAMEFilesize?``
              query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FORCESAMEFilesize?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SAVe:WAVEform:FORCESAMEFilesize value`` command.

        **SCPI Syntax:**

        ::

            - SAVe:WAVEform:FORCESAMEFilesize {<NR1>|OFF|ON}
            - SAVe:WAVEform:FORCESAMEFilesize?

        **Info:**
            - ``<NR1>`` = 0 disables the function; any other value enables it.
            - ``OFF`` disables the function.
            - ``ON`` enables the function.
        """
        return self._forcesamefilesize


class SaveSetup(SCPICmdWrite):
    """The ``SAVe:SETUp`` command.

    **Description:**
        - Stores the state of the oscilloscope to a specified memory location. You can later use the
          ``*RCL`` command to restore the oscilloscope to this saved state.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SAVe:SETUp value`` command.

    **SCPI Syntax:**

    ::

        - SAVe:SETUp {<file path>|<NR1>}

    **Info:**
        - ``<file path>`` is the target location for storing the setup file. <file path> is a quoted
          string that defines the file name and path. Input the file path using the form
          ``<drive>:<dir>``/<filename>. <extension> and one or <dir>s are optional. If you do not
          specify them, the oscilloscope will store the file in the current working directory.
          <filename> stands for a filename. (Use of wildcard characters in filenames is not
          supported.) Filename extensions are not required but are highly recommended. For setups,
          use the extension '.SET'.
        - ``<NR1>`` specifies a location for saving the current front-panel setup. The front-panel
          setup value ranges from 1 to 10. Using an out-of-range value causes an execution error.
          Any settings that have been stored previously at this location will be overwritten.
    """


class SaveMask(SCPICmdWrite):
    """The ``SAVe:MASK`` command.

    **Description:**
        - This command saves the current mask definition to the file specified with a quoted string,
          into the current working directory. You can recall the mask from the file by using the
          command ``RECALL:MASK A`` series of examples showing how to use mask commands for typical
          tasks is included in an appendix.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SAVe:MASK value`` command.

    **SCPI Syntax:**

    ::

        - SAVe:MASK <QString>

    **Info:**
        - ``QString`` is a quoted string that is the name of the mask definition being saved to the
          current working directory.
    """

    _WRAP_ARG_WITH_QUOTES = True


class SaveMarks(SCPICmdWrite):
    """The ``SAVe:MARKS`` command.

    **Description:**
        - This command saves the user search marks in CSV format, to the file that you specify by
          <string>.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SAVe:MARKS value`` command.

    **SCPI Syntax:**

    ::

        - SAVe:MARKS <string>
    """


class SaveEventtableBusItem(ValidatedDynamicNumberCmd, SCPICmdWrite):
    """The ``SAVe:EVENTtable:BUS<x>`` command.

    **Description:**
        - Saves the event table contents of the specified bus in CSV format, to the file that you
          specify by <string>.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:BUS<x> value`` command.

    **SCPI Syntax:**

    ::

        - SAVe:EVENTtable:BUS<x> <string>
    """


class SaveEventtable(SCPICmdRead):
    """The ``SAVe:EVENTtable`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SAVe:EVENTtable?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:EVENTtable?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bus``: The ``SAVe:EVENTtable:BUS<x>`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bus: Dict[int, SaveEventtableBusItem] = DefaultDictPassKeyToFactory(
            lambda x: SaveEventtableBusItem(device, f"{self._cmd_syntax}:BUS{x}")
        )

    @property
    def bus(self) -> Dict[int, SaveEventtableBusItem]:
        """Return the ``SAVe:EVENTtable:BUS<x>`` command.

        **Description:**
            - Saves the event table contents of the specified bus in CSV format, to the file that
              you specify by <string>.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:BUS<x> value``
              command.

        **SCPI Syntax:**

        ::

            - SAVe:EVENTtable:BUS<x> <string>
        """
        return self._bus


class Save(SCPICmdRead):
    """The ``SAVe`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SAVe?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.eventtable``: The ``SAVe:EVENTtable`` command tree.
        - ``.marks``: The ``SAVe:MARKS`` command.
        - ``.mask``: The ``SAVe:MASK`` command.
        - ``.setup``: The ``SAVe:SETUp`` command.
        - ``.waveform``: The ``SAVe:WAVEform`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SAVe") -> None:
        super().__init__(device, cmd_syntax)
        self._eventtable = SaveEventtable(device, f"{self._cmd_syntax}:EVENTtable")
        self._marks = SaveMarks(device, f"{self._cmd_syntax}:MARKS")
        self._mask = SaveMask(device, f"{self._cmd_syntax}:MASK")
        self._setup = SaveSetup(device, f"{self._cmd_syntax}:SETUp")
        self._waveform = SaveWaveform(device, f"{self._cmd_syntax}:WAVEform")

    @property
    def eventtable(self) -> SaveEventtable:
        """Return the ``SAVe:EVENTtable`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SAVe:EVENTtable?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:EVENTtable?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.bus``: The ``SAVe:EVENTtable:BUS<x>`` command.
        """
        return self._eventtable

    @property
    def marks(self) -> SaveMarks:
        """Return the ``SAVe:MARKS`` command.

        **Description:**
            - This command saves the user search marks in CSV format, to the file that you specify
              by <string>.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SAVe:MARKS value`` command.

        **SCPI Syntax:**

        ::

            - SAVe:MARKS <string>
        """
        return self._marks

    @property
    def mask(self) -> SaveMask:
        """Return the ``SAVe:MASK`` command.

        **Description:**
            - This command saves the current mask definition to the file specified with a quoted
              string, into the current working directory. You can recall the mask from the file by
              using the command ``RECALL:MASK A`` series of examples showing how to use mask
              commands for typical tasks is included in an appendix.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SAVe:MASK value`` command.

        **SCPI Syntax:**

        ::

            - SAVe:MASK <QString>

        **Info:**
            - ``QString`` is a quoted string that is the name of the mask definition being saved to
              the current working directory.
        """
        return self._mask

    @property
    def setup(self) -> SaveSetup:
        """Return the ``SAVe:SETUp`` command.

        **Description:**
            - Stores the state of the oscilloscope to a specified memory location. You can later use
              the ``*RCL`` command to restore the oscilloscope to this saved state.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SAVe:SETUp value`` command.

        **SCPI Syntax:**

        ::

            - SAVe:SETUp {<file path>|<NR1>}

        **Info:**
            - ``<file path>`` is the target location for storing the setup file. <file path> is a
              quoted string that defines the file name and path. Input the file path using the form
              ``<drive>:<dir>``/<filename>. <extension> and one or <dir>s are optional. If you do
              not specify them, the oscilloscope will store the file in the current working
              directory. <filename> stands for a filename. (Use of wildcard characters in filenames
              is not supported.) Filename extensions are not required but are highly recommended.
              For setups, use the extension '.SET'.
            - ``<NR1>`` specifies a location for saving the current front-panel setup. The
              front-panel setup value ranges from 1 to 10. Using an out-of-range value causes an
              execution error. Any settings that have been stored previously at this location will
              be overwritten.
        """
        return self._setup

    @property
    def waveform(self) -> SaveWaveform:
        """Return the ``SAVe:WAVEform`` command.

        **Description:**
            - This command (no query form) saves a waveform to one of four reference memory
              locations or a file.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform value`` command.

        **SCPI Syntax:**

        ::

            - SAVe:WAVEform [<wfm>,REF<x>]| [<wfm>,<QString>] | [ALL,[QString]]| [DIGITALALL,<QString>]

        **Info:**
            - ``<wfm>,REF<x>`` saves the specified waveform to the specified reference memory
              location.
            - ``<wfm>,<QString>`` saves the specified waveform to the file specified.
            - ``ALL,[QString>]`` saves all displayed waveforms, excluding digital waveforms, to
              individual files based on the ``SAVE:WAVEFORM:FILEFORMAT`` setting.
            - ``DIGITALALL,<QString>`` saves all digital waveforms to a single file specified by the
              quoted string argument and based on the ``SAVE:WAVEFORM:FILEFORMAT`` setting.

        Sub-properties:
            - ``.data``: The ``SAVe:WAVEform:DATa`` command tree.
            - ``.fileformat``: The ``SAVe:WAVEform:FILEFormat`` command.
            - ``.forcesamefilesize``: The ``SAVe:WAVEform:FORCESAMEFilesize`` command.
        """  # noqa: E501
        return self._waveform



"""The touchscreen commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - TOUCHSCReen:STATe {0|1|OFF|ON}
    - TOUCHSCReen:STATe?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class TouchscreenState(SCPICmdWrite, SCPICmdRead):
    """The ``TOUCHSCReen:STATe`` command.

    **Description:**
        - This sets or queries the enabled state of the touch screen only. This command is
          equivalent to pushing the Touch Off button on the front panel. To completely disable front
          panel operation, use the command LOCK ALL. To re-enable the front panel, send the command
          LOCK NONE.

    **Usage:**
        - Using the ``.query()`` method will send the ``TOUCHSCReen:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``TOUCHSCReen:STATe?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TOUCHSCReen:STATe value`` command.

    **SCPI Syntax:**

    ::

        - TOUCHSCReen:STATe {0|1|OFF|ON}
        - TOUCHSCReen:STATe?

    **Info:**
        - ``0`` disables the touch screen.
        - ``ON`` enables the touch screen.
        - ``OFF`` disables the touch screen.
    """


class Touchscreen(SCPICmdRead):
    """The ``TOUCHSCReen`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``TOUCHSCReen?`` query.
        - Using the ``.verify(value)`` method will send the ``TOUCHSCReen?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``TOUCHSCReen:STATe`` command.
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "TOUCHSCReen"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._state = TouchscreenState(device, f"{self._cmd_syntax}:STATe")

    @property
    def state(self) -> TouchscreenState:
        """Return the ``TOUCHSCReen:STATe`` command.

        **Description:**
            - This sets or queries the enabled state of the touch screen only. This command is
              equivalent to pushing the Touch Off button on the front panel. To completely disable
              front panel operation, use the command LOCK ALL. To re-enable the front panel, send
              the command LOCK NONE.

        **Usage:**
            - Using the ``.query()`` method will send the ``TOUCHSCReen:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``TOUCHSCReen:STATe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TOUCHSCReen:STATe value`` command.

        **SCPI Syntax:**

        ::

            - TOUCHSCReen:STATe {0|1|OFF|ON}
            - TOUCHSCReen:STATe?

        **Info:**
            - ``0`` disables the touch screen.
            - ``ON`` enables the touch screen.
            - ``OFF`` disables the touch screen.
        """
        return self._state



# pylint: disable=too-many-lines
# pylint: disable=line-too-long
"""The visual commands module.

These commands are used in the following models:
LPD6, MSO4, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - VISual:AREA<x>:ASPEctratio {ON|OFF|<NR1>}
    - VISual:AREA<x>:ASPEctratio?
    - VISual:AREA<x>:FLIP:HORizontal
    - VISual:AREA<x>:FLIP:VERTical
    - VISual:AREA<x>:HEIGht <NR3>
    - VISual:AREA<x>:HEIGht?
    - VISual:AREA<x>:HITType {IN|OUT|DONTcare}
    - VISual:AREA<x>:HITType?
    - VISual:AREA<x>:RESET
    - VISual:AREA<x>:ROTAtion <NR3>
    - VISual:AREA<x>:ROTAtion?
    - VISual:AREA<x>:SHAPE {TRIAngle|RECTangle|TRAPezoid|HEXAgon}
    - VISual:AREA<x>:SHAPE?
    - VISual:AREA<x>:SOUrce {CH<x>}
    - VISual:AREA<x>:SOUrce?
    - VISual:AREA<x>:VERTICES '<NR3>, <NR3>, <NR3>, <NR3>, <NR3>, <NR3> [,<NR3>, <NR3>, ...]'
    - VISual:AREA<x>:VERTICES?
    - VISual:AREA<x>:WIDTH <NR3>
    - VISual:AREA<x>:WIDTH?
    - VISual:AREA<x>:XPOSition <NR3>
    - VISual:AREA<x>:XPOSition?
    - VISual:AREA<x>:YPOSition <NR3>
    - VISual:AREA<x>:YPOSition?
    - VISual:DELETEALL
    - VISual:ENAble {ON|OFF|<NR1>}
    - VISual:ENAble?
    - VISual:EQUation <Qstring>
    - VISual:EQUation?
    - VISual:SHOWAReas {ON|OFF|<NR1>}
    - VISual:SHOWAReas?
    - VISual:SHOWCRiteria {ON|OFF|<NR1>}
    - VISual:SHOWCRiteria?
    - VISual:SHOWEQuation {ON|OFF|<NR1>}
    - VISual:SHOWEQuation?
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    SCPICmdWriteNoArguments,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class VisualShowequation(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:SHOWEQuation`` command.

    **Description:**
        - Shows or hides the Visual Trigger area combination logic equation.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:SHOWEQuation?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:SHOWEQuation?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:SHOWEQuation value`` command.

    **SCPI Syntax:**

    ::

        - VISual:SHOWEQuation {ON|OFF|<NR1>}
        - VISual:SHOWEQuation?

    **Info:**
        - ``ON`` shows the Visual Trigger area combination logic equation.
        - ``OFF`` hides the Visual Trigger area combination logic equation.
        - ``<NR1>`` is an integer number. 0 hides the area combination logic equation; any other
          value displays the area combination logic equation.
    """


class VisualShowcriteria(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:SHOWCRiteria`` command.

    **Description:**
        - Sets or queries display of the area names and hit criteria for all visual trigger areas.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:SHOWCRiteria?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:SHOWCRiteria?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:SHOWCRiteria value`` command.

    **SCPI Syntax:**

    ::

        - VISual:SHOWCRiteria {ON|OFF|<NR1>}
        - VISual:SHOWCRiteria?

    **Info:**
        - ``ON`` enables display of the area name and hit criteria labels (In, Out, Don't care
          icons) of all Visual Trigger areas.
        - ``OFF`` hides the area name and hit criteria labels (In, Out, Don't care icons) of all
          Visual Trigger areas.
        - ``<NR1>`` is an integer number. 0 hides the area name and hit criteria of all Visual
          Trigger areas; any other value enables displaying the area name and hit criteria of all
          Visual Trigger areas.
    """


class VisualShowareas(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:SHOWAReas`` command.

    **Description:**
        - Shows or hides all Visual Trigger areas.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:SHOWAReas?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:SHOWAReas?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:SHOWAReas value`` command.

    **SCPI Syntax:**

    ::

        - VISual:SHOWAReas {ON|OFF|<NR1>}
        - VISual:SHOWAReas?

    **Info:**
        - ``ON`` shows all Visual Trigger areas.
        - ``OFF`` hides all Visual Trigger areas.
        - ``<NR1>`` is an integer number. 0 hides all Visual Trigger areas; any other value shows
          all Visual Trigger areas.
    """


class VisualEquation(SCPICmdWrite, SCPICmdRead):
    r"""The ``VISual:EQUation`` command.

    **Description:**
        - Sets or queries the Visual Trigger area combination logic equation.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:EQUation?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:EQUation?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:EQUation value`` command.

    **SCPI Syntax:**

    ::

        - VISual:EQUation <Qstring>
        - VISual:EQUation?

    **Info:**
        - ``<Qstring>`` defines the Visual Trigger area combination logic equation. The equation is
          made up of area names (A<x>) combined with logic operators AND, OR, or XOR (&, \|, ^). It
          may also contain parentheses for grouping. The equation must be true to have a valid
          Visual Trigger event and display a waveform. Each area's true state depends on the area's
          condition setting (In, Out or Don't Care).
    """


class VisualEnable(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:ENAble`` command.

    **Description:**
        - Sets or queries the status (on or off) of the Visual Trigger mode.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:ENAble?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:ENAble value`` command.

    **SCPI Syntax:**

    ::

        - VISual:ENAble {ON|OFF|<NR1>}
        - VISual:ENAble?

    **Info:**
        - ``ON`` enables the Visual Trigger mode.
        - ``OFF`` disables the Visual Trigger mode.
        - ``<NR1>`` is an integer number. 0 turns off the Visual Trigger mode; any other value
          enables Visual Trigger mode.
    """


class VisualDeleteall(SCPICmdWriteNoArguments):
    """The ``VISual:DELETEALL`` command.

    **Description:**
        - Deletes all Visual Trigger areas.

    **Usage:**
        - Using the ``.write()`` method will send the ``VISual:DELETEALL`` command.

    **SCPI Syntax:**

    ::

        - VISual:DELETEALL
    """


class VisualAreaItemYposition(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:YPOSition`` command.

    **Description:**
        - Sets or queries the vertical (Y-axis) center position of the specified Visual Trigger
          area.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:YPOSition?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:YPOSition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:YPOSition value``
          command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:YPOSition <NR3>
        - VISual:AREA<x>:YPOSition?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``<NR3>`` specifies the vertical position of the center of the Visual Trigger area, in
          amplitude (volts, amps).
    """


class VisualAreaItemXposition(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:XPOSition`` command.

    **Description:**
        - Sets or queries the horizontal (X-axis) center position of the specified Visual Trigger
          area.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:XPOSition?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:XPOSition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:XPOSition value``
          command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:XPOSition <NR3>
        - VISual:AREA<x>:XPOSition?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``<NR3>`` specifies the horizontal position of the center of the Visual Trigger area, in
          seconds.
    """


class VisualAreaItemWidth(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:WIDTH`` command.

    **Description:**
        - Sets or queries the width of the specified Visual Trigger area.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:WIDTH?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:WIDTH?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:WIDTH value`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:WIDTH <NR3>
        - VISual:AREA<x>:WIDTH?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``<NR3>`` specifies the width of the Visual Trigger area in seconds.
    """


class VisualAreaItemVertices(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:VERTICES`` command.

    **Description:**
        - Sets or queries the X and Y vertex coordinate values for all vertices of the specified
          Visual Trigger area. Vertex values must be set in pairs.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:VERTICES?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:VERTICES?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:VERTICES value``
          command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:VERTICES '<NR3>, <NR3>, <NR3>, <NR3>, <NR3>, <NR3> [,<NR3>, <NR3>, ...]'
        - VISual:AREA<x>:VERTICES?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``<NR3>, <NR3>`` specifies the X, Y coordinate pair of each vertex in an area. The first
          value is X (time) and the second value is Y (amplitude). There must be a minimum of three
          vertex pairs to create an area. If the specified area exists, the area is changed to the
          shape specified in the arguments. If the specified area does not exist, a new area is
          created and assigned the specified vertices.
    """


class VisualAreaItemSource(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:SOUrce`` command.

    **Description:**
        - Sets or queries the signal source for the specified Visual Trigger area. The source can
          only be an analog channel.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SOUrce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SOUrce value`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:SOUrce {CH<x>}
        - VISual:AREA<x>:SOUrce?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``CH1`` through CH8 sets the source channel number for the specified area.
    """


class VisualAreaItemShape(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:SHAPE`` command.

    **Description:**
        - Sets or queries the current shape of the area.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:SHAPE?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SHAPE?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SHAPE value`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:SHAPE {TRIAngle|RECTangle|TRAPezoid|HEXAgon}
        - VISual:AREA<x>:SHAPE?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``TRIAngle`` sets the specified area to a triangular shape (three vertices). If the area
          does not exist, the instrument creates a new triangular area with the specified area
          number.
        - ``RECTangle`` sets the specified area to a rectangular shape (four vertices, right angles
          at each corner). If the area does not exist, the instrument creates a new triangular area
          with the specified area number.
        - ``TRAPezoid`` sets the specified area to a trapezoidal shape (four vertices, two parallel
          sides). If the area does not exist, the instrument creates a new triangular area with the
          specified area number.
        - ``HEXAgon`` sets the specified area to a hexagonal shape (six vertices). If the area does
          not exist, the instrument creates a new hexagonal area with the specified area number.
    """


class VisualAreaItemRotation(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:ROTAtion`` command.

    **Description:**
        - Sets or queries the rotation angle of the specified Visual Trigger area.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:ROTAtion?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:ROTAtion?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:ROTAtion value``
          command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:ROTAtion <NR3>
        - VISual:AREA<x>:ROTAtion?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``<NR3>`` specifies the rotation angle of the Visual Trigger area, in positive degrees
          from 0 to 360. Zero degrees is referenced from when the area was created.
    """


class VisualAreaItemReset(SCPICmdWriteNoArguments):
    """The ``VISual:AREA<x>:RESET`` command.

    **Description:**
        - Sets the specified Visual Trigger area shape to a default-sized triangle.

    **Usage:**
        - Using the ``.write()`` method will send the ``VISual:AREA<x>:RESET`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:RESET

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
    """


class VisualAreaItemHittype(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:HITType`` command.

    **Description:**
        - Sets or queries the area hit logic true condition for the specified Visual Trigger area.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:HITType?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:HITType?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:HITType value`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:HITType {IN|OUT|DONTcare}
        - VISual:AREA<x>:HITType?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``IN`` specifies that the waveform must intrude anywhere into the specified area to be
          true.
        - ``OUT`` specifies that the waveform must not intrude anywhere into the specified area to
          be true.
        - ``DONTcare`` sets the area to a don't care state, causing the area to be ignored. This is
          useful when you are developing a Visual Trigger condition and need to change the hit logic
          type of an area while keeping the area.
    """


class VisualAreaItemHeight(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:HEIGht`` command.

    **Description:**
        - Sets or queries the height of the specified Visual Trigger area.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:HEIGht?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:HEIGht?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:HEIGht value`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:HEIGht <NR3>
        - VISual:AREA<x>:HEIGht?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``<NR3>`` specifies the height of the Visual Trigger area in amplitude.
    """


class VisualAreaItemFlipVertical(SCPICmdWriteNoArguments):
    """The ``VISual:AREA<x>:FLIP:VERTical`` command.

    **Description:**
        - Flips the specified Visual Trigger area vertically around its center point.

    **Usage:**
        - Using the ``.write()`` method will send the ``VISual:AREA<x>:FLIP:VERTical`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:FLIP:VERTical

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
    """


class VisualAreaItemFlipHorizontal(SCPICmdWriteNoArguments):
    """The ``VISual:AREA<x>:FLIP:HORizontal`` command.

    **Description:**
        - Flips the specified Visual Trigger area horizontally around its center point.

    **Usage:**
        - Using the ``.write()`` method will send the ``VISual:AREA<x>:FLIP:HORizontal`` command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:FLIP:HORizontal

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
    """


class VisualAreaItemFlip(SCPICmdRead):
    """The ``VISual:AREA<x>:FLIP`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:FLIP?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:FLIP?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.horizontal``: The ``VISual:AREA<x>:FLIP:HORizontal`` command.
        - ``.vertical``: The ``VISual:AREA<x>:FLIP:VERTical`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._horizontal = VisualAreaItemFlipHorizontal(device, f"{self._cmd_syntax}:HORizontal")
        self._vertical = VisualAreaItemFlipVertical(device, f"{self._cmd_syntax}:VERTical")

    @property
    def horizontal(self) -> VisualAreaItemFlipHorizontal:
        """Return the ``VISual:AREA<x>:FLIP:HORizontal`` command.

        **Description:**
            - Flips the specified Visual Trigger area horizontally around its center point.

        **Usage:**
            - Using the ``.write()`` method will send the ``VISual:AREA<x>:FLIP:HORizontal``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:FLIP:HORizontal

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        """
        return self._horizontal

    @property
    def vertical(self) -> VisualAreaItemFlipVertical:
        """Return the ``VISual:AREA<x>:FLIP:VERTical`` command.

        **Description:**
            - Flips the specified Visual Trigger area vertically around its center point.

        **Usage:**
            - Using the ``.write()`` method will send the ``VISual:AREA<x>:FLIP:VERTical`` command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:FLIP:VERTical

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        """
        return self._vertical


class VisualAreaItemAspectratio(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:ASPEctratio`` command.

    **Description:**
        - Sets or queries whether the aspect ratio of the specified Visual Trigger area is locked.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:ASPEctratio?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:ASPEctratio?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:ASPEctratio value``
          command.

    **SCPI Syntax:**

    ::

        - VISual:AREA<x>:ASPEctratio {ON|OFF|<NR1>}
        - VISual:AREA<x>:ASPEctratio?

    **Info:**
        - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        - ``ON`` locks the aspect ratio of the specified Visual Trigger area. The aspect ratio is
          kept constant when the height or width of the area changes.
        - ``OFF`` unlocks the aspect ratio of the specified Visual Trigger area.
        - ``<NR1>`` is an integer number. 1 locks the aspect ratio of the specified Visual Trigger
          area; any other value unlocks the aspect ratio of the specified Visual Trigger area.
    """


#  pylint: disable=too-many-instance-attributes
class VisualAreaItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``VISual:AREA<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual:AREA<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.aspectratio``: The ``VISual:AREA<x>:ASPEctratio`` command.
        - ``.flip``: The ``VISual:AREA<x>:FLIP`` command tree.
        - ``.height``: The ``VISual:AREA<x>:HEIGht`` command.
        - ``.hittype``: The ``VISual:AREA<x>:HITType`` command.
        - ``.reset``: The ``VISual:AREA<x>:RESET`` command.
        - ``.rotation``: The ``VISual:AREA<x>:ROTAtion`` command.
        - ``.shape``: The ``VISual:AREA<x>:SHAPE`` command.
        - ``.source``: The ``VISual:AREA<x>:SOUrce`` command.
        - ``.vertices``: The ``VISual:AREA<x>:VERTICES`` command.
        - ``.width``: The ``VISual:AREA<x>:WIDTH`` command.
        - ``.xposition``: The ``VISual:AREA<x>:XPOSition`` command.
        - ``.yposition``: The ``VISual:AREA<x>:YPOSition`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._aspectratio = VisualAreaItemAspectratio(device, f"{self._cmd_syntax}:ASPEctratio")
        self._flip = VisualAreaItemFlip(device, f"{self._cmd_syntax}:FLIP")
        self._height = VisualAreaItemHeight(device, f"{self._cmd_syntax}:HEIGht")
        self._hittype = VisualAreaItemHittype(device, f"{self._cmd_syntax}:HITType")
        self._reset = VisualAreaItemReset(device, f"{self._cmd_syntax}:RESET")
        self._rotation = VisualAreaItemRotation(device, f"{self._cmd_syntax}:ROTAtion")
        self._shape = VisualAreaItemShape(device, f"{self._cmd_syntax}:SHAPE")
        self._source = VisualAreaItemSource(device, f"{self._cmd_syntax}:SOUrce")
        self._vertices = VisualAreaItemVertices(device, f"{self._cmd_syntax}:VERTICES")
        self._width = VisualAreaItemWidth(device, f"{self._cmd_syntax}:WIDTH")
        self._xposition = VisualAreaItemXposition(device, f"{self._cmd_syntax}:XPOSition")
        self._yposition = VisualAreaItemYposition(device, f"{self._cmd_syntax}:YPOSition")

    @property
    def aspectratio(self) -> VisualAreaItemAspectratio:
        """Return the ``VISual:AREA<x>:ASPEctratio`` command.

        **Description:**
            - Sets or queries whether the aspect ratio of the specified Visual Trigger area is
              locked.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:ASPEctratio?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:ASPEctratio?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:ASPEctratio value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:ASPEctratio {ON|OFF|<NR1>}
            - VISual:AREA<x>:ASPEctratio?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``ON`` locks the aspect ratio of the specified Visual Trigger area. The aspect ratio
              is kept constant when the height or width of the area changes.
            - ``OFF`` unlocks the aspect ratio of the specified Visual Trigger area.
            - ``<NR1>`` is an integer number. 1 locks the aspect ratio of the specified Visual
              Trigger area; any other value unlocks the aspect ratio of the specified Visual Trigger
              area.
        """
        return self._aspectratio

    @property
    def flip(self) -> VisualAreaItemFlip:
        """Return the ``VISual:AREA<x>:FLIP`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:FLIP?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:FLIP?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.horizontal``: The ``VISual:AREA<x>:FLIP:HORizontal`` command.
            - ``.vertical``: The ``VISual:AREA<x>:FLIP:VERTical`` command.
        """
        return self._flip

    @property
    def height(self) -> VisualAreaItemHeight:
        """Return the ``VISual:AREA<x>:HEIGht`` command.

        **Description:**
            - Sets or queries the height of the specified Visual Trigger area.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:HEIGht?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:HEIGht?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:HEIGht value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:HEIGht <NR3>
            - VISual:AREA<x>:HEIGht?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``<NR3>`` specifies the height of the Visual Trigger area in amplitude.
        """
        return self._height

    @property
    def hittype(self) -> VisualAreaItemHittype:
        """Return the ``VISual:AREA<x>:HITType`` command.

        **Description:**
            - Sets or queries the area hit logic true condition for the specified Visual Trigger
              area.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:HITType?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:HITType?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:HITType value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:HITType {IN|OUT|DONTcare}
            - VISual:AREA<x>:HITType?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``IN`` specifies that the waveform must intrude anywhere into the specified area to be
              true.
            - ``OUT`` specifies that the waveform must not intrude anywhere into the specified area
              to be true.
            - ``DONTcare`` sets the area to a don't care state, causing the area to be ignored. This
              is useful when you are developing a Visual Trigger condition and need to change the
              hit logic type of an area while keeping the area.
        """
        return self._hittype

    @property
    def reset(self) -> VisualAreaItemReset:
        """Return the ``VISual:AREA<x>:RESET`` command.

        **Description:**
            - Sets the specified Visual Trigger area shape to a default-sized triangle.

        **Usage:**
            - Using the ``.write()`` method will send the ``VISual:AREA<x>:RESET`` command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:RESET

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
        """
        return self._reset

    @property
    def rotation(self) -> VisualAreaItemRotation:
        """Return the ``VISual:AREA<x>:ROTAtion`` command.

        **Description:**
            - Sets or queries the rotation angle of the specified Visual Trigger area.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:ROTAtion?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:ROTAtion?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:ROTAtion value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:ROTAtion <NR3>
            - VISual:AREA<x>:ROTAtion?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``<NR3>`` specifies the rotation angle of the Visual Trigger area, in positive degrees
              from 0 to 360. Zero degrees is referenced from when the area was created.
        """
        return self._rotation

    @property
    def shape(self) -> VisualAreaItemShape:
        """Return the ``VISual:AREA<x>:SHAPE`` command.

        **Description:**
            - Sets or queries the current shape of the area.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:SHAPE?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SHAPE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SHAPE value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:SHAPE {TRIAngle|RECTangle|TRAPezoid|HEXAgon}
            - VISual:AREA<x>:SHAPE?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``TRIAngle`` sets the specified area to a triangular shape (three vertices). If the
              area does not exist, the instrument creates a new triangular area with the specified
              area number.
            - ``RECTangle`` sets the specified area to a rectangular shape (four vertices, right
              angles at each corner). If the area does not exist, the instrument creates a new
              triangular area with the specified area number.
            - ``TRAPezoid`` sets the specified area to a trapezoidal shape (four vertices, two
              parallel sides). If the area does not exist, the instrument creates a new triangular
              area with the specified area number.
            - ``HEXAgon`` sets the specified area to a hexagonal shape (six vertices). If the area
              does not exist, the instrument creates a new hexagonal area with the specified area
              number.
        """
        return self._shape

    @property
    def source(self) -> VisualAreaItemSource:
        """Return the ``VISual:AREA<x>:SOUrce`` command.

        **Description:**
            - Sets or queries the signal source for the specified Visual Trigger area. The source
              can only be an analog channel.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SOUrce?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SOUrce value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:SOUrce {CH<x>}
            - VISual:AREA<x>:SOUrce?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``CH1`` through CH8 sets the source channel number for the specified area.
        """
        return self._source

    @property
    def vertices(self) -> VisualAreaItemVertices:
        """Return the ``VISual:AREA<x>:VERTICES`` command.

        **Description:**
            - Sets or queries the X and Y vertex coordinate values for all vertices of the specified
              Visual Trigger area. Vertex values must be set in pairs.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:VERTICES?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:VERTICES?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:VERTICES value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:VERTICES '<NR3>, <NR3>, <NR3>, <NR3>, <NR3>, <NR3> [,<NR3>, <NR3>, ...]'
            - VISual:AREA<x>:VERTICES?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``<NR3>, <NR3>`` specifies the X, Y coordinate pair of each vertex in an area. The
              first value is X (time) and the second value is Y (amplitude). There must be a minimum
              of three vertex pairs to create an area. If the specified area exists, the area is
              changed to the shape specified in the arguments. If the specified area does not exist,
              a new area is created and assigned the specified vertices.
        """  # noqa: E501
        return self._vertices

    @property
    def width(self) -> VisualAreaItemWidth:
        """Return the ``VISual:AREA<x>:WIDTH`` command.

        **Description:**
            - Sets or queries the width of the specified Visual Trigger area.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:WIDTH?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:WIDTH?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:WIDTH value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:WIDTH <NR3>
            - VISual:AREA<x>:WIDTH?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``<NR3>`` specifies the width of the Visual Trigger area in seconds.
        """
        return self._width

    @property
    def xposition(self) -> VisualAreaItemXposition:
        """Return the ``VISual:AREA<x>:XPOSition`` command.

        **Description:**
            - Sets or queries the horizontal (X-axis) center position of the specified Visual
              Trigger area.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:XPOSition?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:XPOSition?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:XPOSition value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:XPOSition <NR3>
            - VISual:AREA<x>:XPOSition?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``<NR3>`` specifies the horizontal position of the center of the Visual Trigger area,
              in seconds.
        """
        return self._xposition

    @property
    def yposition(self) -> VisualAreaItemYposition:
        """Return the ``VISual:AREA<x>:YPOSition`` command.

        **Description:**
            - Sets or queries the vertical (Y-axis) center position of the specified Visual Trigger
              area.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:YPOSition?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:YPOSition?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:YPOSition value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:AREA<x>:YPOSition <NR3>
            - VISual:AREA<x>:YPOSition?

        **Info:**
            - ``Area<x>`` specifies the integer number of a Visual Trigger area.
            - ``<NR3>`` specifies the vertical position of the center of the Visual Trigger area, in
              amplitude (volts, amps).
        """
        return self._yposition


class Visual(SCPICmdRead):
    """The ``VISual`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``VISual?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.area``: The ``VISual:AREA<x>`` command tree.
        - ``.deleteall``: The ``VISual:DELETEALL`` command.
        - ``.enable``: The ``VISual:ENAble`` command.
        - ``.equation``: The ``VISual:EQUation`` command.
        - ``.showareas``: The ``VISual:SHOWAReas`` command.
        - ``.showcriteria``: The ``VISual:SHOWCRiteria`` command.
        - ``.showequation``: The ``VISual:SHOWEQuation`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "VISual") -> None:
        super().__init__(device, cmd_syntax)
        self._area: Dict[int, VisualAreaItem] = DefaultDictPassKeyToFactory(
            lambda x: VisualAreaItem(device, f"{self._cmd_syntax}:AREA{x}")
        )
        self._deleteall = VisualDeleteall(device, f"{self._cmd_syntax}:DELETEALL")
        self._enable = VisualEnable(device, f"{self._cmd_syntax}:ENAble")
        self._equation = VisualEquation(device, f"{self._cmd_syntax}:EQUation")
        self._showareas = VisualShowareas(device, f"{self._cmd_syntax}:SHOWAReas")
        self._showcriteria = VisualShowcriteria(device, f"{self._cmd_syntax}:SHOWCRiteria")
        self._showequation = VisualShowequation(device, f"{self._cmd_syntax}:SHOWEQuation")

    @property
    def area(self) -> Dict[int, VisualAreaItem]:
        """Return the ``VISual:AREA<x>`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:AREA<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.aspectratio``: The ``VISual:AREA<x>:ASPEctratio`` command.
            - ``.flip``: The ``VISual:AREA<x>:FLIP`` command tree.
            - ``.height``: The ``VISual:AREA<x>:HEIGht`` command.
            - ``.hittype``: The ``VISual:AREA<x>:HITType`` command.
            - ``.reset``: The ``VISual:AREA<x>:RESET`` command.
            - ``.rotation``: The ``VISual:AREA<x>:ROTAtion`` command.
            - ``.shape``: The ``VISual:AREA<x>:SHAPE`` command.
            - ``.source``: The ``VISual:AREA<x>:SOUrce`` command.
            - ``.vertices``: The ``VISual:AREA<x>:VERTICES`` command.
            - ``.width``: The ``VISual:AREA<x>:WIDTH`` command.
            - ``.xposition``: The ``VISual:AREA<x>:XPOSition`` command.
            - ``.yposition``: The ``VISual:AREA<x>:YPOSition`` command.
        """
        return self._area

    @property
    def deleteall(self) -> VisualDeleteall:
        """Return the ``VISual:DELETEALL`` command.

        **Description:**
            - Deletes all Visual Trigger areas.

        **Usage:**
            - Using the ``.write()`` method will send the ``VISual:DELETEALL`` command.

        **SCPI Syntax:**

        ::

            - VISual:DELETEALL
        """
        return self._deleteall

    @property
    def enable(self) -> VisualEnable:
        """Return the ``VISual:ENAble`` command.

        **Description:**
            - Sets or queries the status (on or off) of the Visual Trigger mode.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:ENAble?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:ENAble value`` command.

        **SCPI Syntax:**

        ::

            - VISual:ENAble {ON|OFF|<NR1>}
            - VISual:ENAble?

        **Info:**
            - ``ON`` enables the Visual Trigger mode.
            - ``OFF`` disables the Visual Trigger mode.
            - ``<NR1>`` is an integer number. 0 turns off the Visual Trigger mode; any other value
              enables Visual Trigger mode.
        """
        return self._enable

    @property
    def equation(self) -> VisualEquation:
        r"""Return the ``VISual:EQUation`` command.

        **Description:**
            - Sets or queries the Visual Trigger area combination logic equation.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:EQUation?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:EQUation?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:EQUation value`` command.

        **SCPI Syntax:**

        ::

            - VISual:EQUation <Qstring>
            - VISual:EQUation?

        **Info:**
            - ``<Qstring>`` defines the Visual Trigger area combination logic equation. The equation
              is made up of area names (A<x>) combined with logic operators AND, OR, or XOR (&, \|,
              ^). It may also contain parentheses for grouping. The equation must be true to have a
              valid Visual Trigger event and display a waveform. Each area's true state depends on
              the area's condition setting (In, Out or Don't Care).
        """
        return self._equation

    @property
    def showareas(self) -> VisualShowareas:
        """Return the ``VISual:SHOWAReas`` command.

        **Description:**
            - Shows or hides all Visual Trigger areas.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:SHOWAReas?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:SHOWAReas?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:SHOWAReas value`` command.

        **SCPI Syntax:**

        ::

            - VISual:SHOWAReas {ON|OFF|<NR1>}
            - VISual:SHOWAReas?

        **Info:**
            - ``ON`` shows all Visual Trigger areas.
            - ``OFF`` hides all Visual Trigger areas.
            - ``<NR1>`` is an integer number. 0 hides all Visual Trigger areas; any other value
              shows all Visual Trigger areas.
        """
        return self._showareas

    @property
    def showcriteria(self) -> VisualShowcriteria:
        """Return the ``VISual:SHOWCRiteria`` command.

        **Description:**
            - Sets or queries display of the area names and hit criteria for all visual trigger
              areas.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:SHOWCRiteria?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:SHOWCRiteria?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:SHOWCRiteria value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:SHOWCRiteria {ON|OFF|<NR1>}
            - VISual:SHOWCRiteria?

        **Info:**
            - ``ON`` enables display of the area name and hit criteria labels (In, Out, Don't care
              icons) of all Visual Trigger areas.
            - ``OFF`` hides the area name and hit criteria labels (In, Out, Don't care icons) of all
              Visual Trigger areas.
            - ``<NR1>`` is an integer number. 0 hides the area name and hit criteria of all Visual
              Trigger areas; any other value enables displaying the area name and hit criteria of
              all Visual Trigger areas.
        """
        return self._showcriteria

    @property
    def showequation(self) -> VisualShowequation:
        """Return the ``VISual:SHOWEQuation`` command.

        **Description:**
            - Shows or hides the Visual Trigger area combination logic equation.

        **Usage:**
            - Using the ``.query()`` method will send the ``VISual:SHOWEQuation?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:SHOWEQuation?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:SHOWEQuation value``
              command.

        **SCPI Syntax:**

        ::

            - VISual:SHOWEQuation {ON|OFF|<NR1>}
            - VISual:SHOWEQuation?

        **Info:**
            - ``ON`` shows the Visual Trigger area combination logic equation.
            - ``OFF`` hides the Visual Trigger area combination logic equation.
            - ``<NR1>`` is an integer number. 0 hides the area combination logic equation; any other
              value displays the area combination logic equation.
        """
        return self._showequation



"""MSO5B device driver module."""
from tm_devices.commands import MSO5BMixin
from tm_devices.drivers.pi.scopes.tekscope.mso5 import MSO5


class MSO5B(MSO5BMixin, MSO5):
    """MSO5B device driver."""



"""Base SMU 6000 device driver module."""
from __future__ import annotations

import inspect

from abc import ABC
from functools import cached_property
from typing import Optional, Tuple, TYPE_CHECKING, Union

from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.pi._ieee488_2_commands import IEEE4882Commands
from tm_devices.drivers.pi.pi_device import PIDevice
from tm_devices.drivers.pi.signal_sources.signal_source import SignalSource
from tm_devices.drivers.pi.source_measure_units.source_measure_unit import SourceMeasureUnit

if TYPE_CHECKING:
    import os


@family_base_class
class SMU6000(SourceMeasureUnit, ABC):
    """Base SMU6000 device driver."""

    _IEEE_COMMANDS_CLASS = IEEE4882Commands

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return tuple(f"SOURCE{x+1}" for x in range(self.total_channels))

    @property
    def ieee_cmds(self) -> IEEE4882Commands:  # pyright: ignore [reportIncompatibleMethodOverride]
        """Return an internal class containing methods for the standard IEEE 488.2 command set."""
        return self._ieee_cmds

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        return 1

    ################################################################################################
    # Public Methods
    ################################################################################################
    def expect_esr(self, esr: Union[int, str], error_string: str = "") -> Tuple[bool, str]:
        r"""Check for the expected number of errors and output string.

        Sends the ``*ESR?`` and SYSTEM:ERROR? queries.

        Args:
            esr: Expected ``*ESR?`` value
            error_string: Expected error buffer string.
                Multiple errors should be separated by a \n character

        Returns:
            Boolean indicating if the check passed or failed and a string with the results.
        """
        return SignalSource.expect_esr(self, esr, error_string)  # type: ignore

    def get_eventlog_status(self) -> Tuple[bool, str]:
        """Help function for getting the eventlog status.

        Returns:
            Boolean indicating no error, String containing concatenated contents of event log.
        """
        return SignalSource.get_eventlog_status(self)  # type: ignore

    def run_script(self, script_name: str) -> None:  # noqa: ARG002
        """Not Implemented."""
        msg = f"This functionality is not available on the {self.__class__.__name__} instrument."
        raise NotImplementedError(msg)

    def set_and_check(  # noqa: PLR0913
        self,
        command: str,
        value: Union[str, float],
        tolerance: float = 0,
        percentage: bool = False,
        remove_quotes: bool = False,
        custom_message_prefix: str = "",
        *,
        expected_value: Optional[Union[str, float]] = None,
    ) -> str:
        """Send the given command with the given value and then verify the results.

        Args:
            command: The command to send.
                For example: ``:AFG:FUNCTION``
            value: The value being set by the command.
                For example: ``SINE``
            tolerance: The acceptable difference between two floating point values.
            percentage: A boolean indicating what kind of tolerance check to perform.
                 False means absolute tolerance: +/- tolerance.
                 True means percent tolerance: +/- (tolerance / 100) * value.
            remove_quotes: Set this to True to remove all double quotes from the returned value.
            custom_message_prefix: A custom message to be prepended to the failure message.
            expected_value: An optional, alternative value expected to be returned.

        Returns:
            The output of the query portion of the method.
        """
        return PIDevice.set_and_check(
            self,
            command,
            value,
            tolerance,
            percentage,
            remove_quotes,
            custom_message_prefix,
            expected_value=expected_value,
        )

    def load_script(
        self,
        file_path: Union[str, os.PathLike[str]],  # noqa: ARG002
        script_name: str = "",  # noqa: ARG002
        *,
        run_script: bool = False,  # noqa: ARG002
        to_nv_memory: bool = False,  # noqa: ARG002
    ) -> None:
        """Not Implemented."""
        msg = f"This functionality is not available on the {self.__class__.__name__} instrument."
        raise NotImplementedError(msg)

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _reboot(self) -> None:
        """Perform the actual rebooting code.

        Raises:
            NotImplementedError: Indicates the current driver has not implemented this method.
        """
        # TODO: implement
        raise NotImplementedError(
            f"``.{inspect.currentframe().f_code.co_name}()``"  # pyright: ignore
            f" is not yet implemented for the {self.__class__.__name__} driver"
        )



"""Base DMM7500 device driver module."""
from abc import ABC
from typing import Tuple

from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.pi._ieee488_2_commands import LegacyTSPIEEE4882Commands
from tm_devices.drivers.pi.digital_multimeters.digital_multimeter import DigitalMultimeter


@family_base_class
class DMM7500(DigitalMultimeter, ABC):
    """Base DMM7500 device driver."""

    _IEEE_COMMANDS_CLASS = LegacyTSPIEEE4882Commands

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return ()

    @property
    def ieee_cmds(self) -> LegacyTSPIEEE4882Commands:
        """Return an internal class containing methods for the standard IEEE 488.2 command set."""
        return self._ieee_cmds  # type: ignore

    ################################################################################################
    # Public Methods
    ################################################################################################
    def get_eventlog_status(self) -> Tuple[bool, str]:
        """Help function for getting the eventlog status.

        Returns:
            Boolean indicating no error, String containing concatenated contents of event log.
        """
        result_allev = False
        allev_result_str = '0,"No events to report - queue empty"'

        if not (err_count := int(self.query("print(eventlog.getcount(eventlog.SEV_ERROR))"))):
            result_allev = True
        else:
            allev_result_list = [self.query("print(eventlog.next())") for _ in range(err_count)]
            allev_result_str = ",".join(allev_result_list)
        return result_allev, allev_result_str

    ################################################################################################
    # Private Methods
    ################################################################################################



"""Data Acquisition (DAQ) and Logging, Multimeter Systems package init file."""
from tm_devices.drivers.pi.data_acquisition_systems.daq6510 import DAQ6510
from tm_devices.drivers.pi.data_acquisition_systems.data_acquisition_system import (
    DataAcquisitionSystem,
)

__all__ = [
    "DataAcquisitionSystem",
    "DAQ6510",
]
